#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class ServerGroupsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(ServerGroupsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-server-groups', 'compute'):
            msg = "%s skipped as os-server-groups not enabled." % cls.__name__
            raise cls.skipException(msg)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-groups:create"])
    @decorators.idempotent_id('7f3eae94-6130-47e9-81ac-34009f55be2f')
    def test_create_server_group(self):
        with self.override_role():
            self.create_test_server_group()

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-groups:delete"])
    @decorators.idempotent_id('832d9be3-632e-47b2-93d2-5897db43e3e2')
    def test_delete_server_group(self):
        server_group = self.create_test_server_group()
        with self.override_role():
            self.server_groups_client.delete_server_group(server_group['id'])

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-groups:index"])
    @decorators.idempotent_id('5eccd67f-5945-483b-b1c8-de851ebfc1c1')
    def test_list_server_groups(self):
        with self.override_role():
            self.server_groups_client.list_server_groups()

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-groups:show"])
    @decorators.idempotent_id('62534e3f-7e99-4a3d-a08e-33e056460cf2')
    def test_show_server_group(self):
        server_group = self.create_test_server_group()
        with self.override_role():
            self.server_groups_client.show_server_group(server_group['id'])



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class ServerTagsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    min_microversion = '2.26'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(ServerTagsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-server-tags', 'compute'):
            msg = "os-server-tags extension is not enabled."
            raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerTagsRbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerTagsRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    def _add_tag_to_server(self):
        tag_name = data_utils.rand_name(self.__class__.__name__ + '-tag')
        self.servers_client.update_tag(self.server['id'], tag_name)
        self.addCleanup(self.servers_client.delete_all_tags, self.server['id'])
        return tag_name

    @decorators.idempotent_id('99e73dd3-adec-4044-b46c-84bdded35d09')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:index"])
    def test_list_tags(self):
        with self.override_role():
            self.servers_client.list_tags(self.server['id'])

    @decorators.idempotent_id('9297c99e-94eb-429f-93cf-9b1838e33622')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:show"])
    def test_check_tag_existence(self):
        tag_name = self._add_tag_to_server()
        with self.override_role():
            self.servers_client.check_tag_existence(self.server['id'],
                                                    tag_name)

    @decorators.idempotent_id('0d84ee94-d3ca-4635-8edf-b7f67ab8e4a3')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:update"])
    def test_update_tag(self):
        with self.override_role():
            self._add_tag_to_server()

    @decorators.idempotent_id('115c2694-00aa-41ee-99f6-9eab4040c182')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:delete"])
    def test_delete_tag(self):
        tag_name = self._add_tag_to_server()
        with self.override_role():
            self.servers_client.delete_tag(self.server['id'], tag_name)

    @decorators.idempotent_id('a8e19b87-6580-4bc8-9933-e62561ff667d')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:update_all"])
    def test_update_all_tags(self):
        new_tag_name = data_utils.rand_name(self.__class__.__name__ + '-tag')
        with self.override_role():
            self.servers_client.update_all_tags(self.server['id'],
                                                [new_tag_name])

    @decorators.idempotent_id('89d51936-e333-42f9-a045-132a4865ba1a')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-tags:delete_all"])
    def test_delete_all_tags(self):
        with self.override_role():
            self.servers_client.delete_all_tags(self.server['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

import six


@six.add_metaclass(abc.ABCMeta)
class RbacAuthority(object):
    """Class for validating whether a given role can perform a policy action.

    Any class that extends ``RbacAuthority`` provides the logic for determining
    whether a role has permissions to execute a policy action.
    """

    @abc.abstractmethod
    def allowed(self, rule, role):
        """Determine whether the role should be able to perform the API.

        :param rule: The name of the policy enforced by the API.
        :param role: The role used to determine whether ``rule`` can be
            executed.
        :returns: True if the ``role`` has permissions to execute
            ``rule``, else False.
        """



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin import rbac_utils
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityTokenV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    credentials = ['primary', 'alt']

    @classmethod
    def skip_checks(cls):
        super(IdentityTokenV3RbacTest, cls).skip_checks()
        # In case of admin, the positive testcase would be used, hence
        # skipping negative testcase.
        if rbac_utils.is_admin():
            raise cls.skipException(
                "Skipped as admin role doesn't require negative testing")

    def _setup_alt_token(self):
        return self.setup_test_token(
            self.os_alt.auth_provider.credentials.user_id,
            self.os_alt.auth_provider.credentials.password)

    @decorators.idempotent_id('c83c8f1a-79cb-4dc4-b55f-c7d2bfd98b1e')
    @decorators.attr(type=['negative'])
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:validate_token"],
        extra_target_data={
            "target.token.user_id":
            "os_alt.auth_provider.credentials.user_id"
        })
    def test_show_token_negative(self):
        # Explicit negative test for identity:validate_token policy action.
        # Assert expected exception is Forbidden and then reraise it.
        alt_token_id = self._setup_alt_token()
        with self.override_role():
            e = self.assertRaises(lib_exc.Forbidden,
                                  self.identity_client.show_token,
                                  alt_token_id)
            raise e

    @decorators.idempotent_id('2786a55d-a818-433a-af7a-41ebf72ab4da')
    @decorators.attr(type=['negative'])
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:revoke_token"],
        extra_target_data={
            "target.token.user_id":
            "os_alt.auth_provider.credentials.user_id"
        })
    def test_delete_token_negative(self):
        # Explicit negative test for identity:revoke_token policy action.
        # Assert expected exception is Forbidden and then reraise it.
        alt_token_id = self._setup_alt_token()
        with self.override_role():
            e = self.assertRaises(lib_exc.Forbidden,
                                  self.identity_client.delete_token,
                                  alt_token_id)
            raise e

    @decorators.idempotent_id('1ea02ac0-9a96-44bd-bdc3-4dae3c10cc2e')
    @decorators.attr(type=['negative'])
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:check_token"],
        extra_target_data={
            "target.token.user_id":
            "os_alt.auth_provider.credentials.user_id"
        })
    def test_check_token_existence_negative(self):
        # Explicit negative test for identity:check_token policy action.
        # Assert expected exception is Forbidden and then reraise it.
        alt_token_id = self._setup_alt_token()
        with self.override_role():
            e = self.assertRaises(lib_exc.Forbidden,
                                  self.identity_client.check_token_existence,
                                  alt_token_id)
            raise e



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityEndpointsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_endpoint"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd127')
    def test_create_endpoint(self):
        service = self.setup_test_service()
        with self.override_role():
            self.setup_test_endpoint(service=service)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_endpoint"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd128')
    def test_update_endpoint(self):
        endpoint = self.setup_test_endpoint()
        new_url = data_utils.rand_url()

        with self.override_role():
            self.endpoints_client.update_endpoint(
                endpoint["id"],
                url=new_url)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_endpoint"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd129')
    def test_delete_endpoint(self):
        endpoint = self.setup_test_endpoint()

        with self.override_role():
            self.endpoints_client.delete_endpoint(endpoint['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_endpoint"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd130')
    def test_show_endpoint(self):
        endpoint = self.setup_test_endpoint()

        with self.override_role():
            self.endpoints_client.show_endpoint(endpoint['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_endpoints"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd131')
    def test_list_endpoints(self):
        with self.override_role():
            self.endpoints_client.list_endpoints()



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base


class ImageNamespacesObjectsRbacTest(rbac_base.BaseV2ImageRbacTest):

    @rbac_rule_validation.action(service="glance",
                                 rules=["add_metadef_object"])
    @decorators.idempotent_id("772156f2-e33d-432e-8521-12385746c2f0")
    def test_create_metadef_object_in_namespace(self):
        """Create Metadef Object Namespace Test

        RBAC test for the glance add_metadef_object policy
        """
        namespace = self.create_namespace()
        # create a md object, it will be cleaned automatically after
        # cleanup of namespace
        object_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-object')
        with self.override_role():
            self.namespace_objects_client.create_namespace_object(
                namespace['namespace'],
                name=object_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_objects_client.delete_namespace_object,
                        namespace['namespace'], object_name)

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_metadef_objects"])
    @decorators.idempotent_id("48b50ecb-237d-4909-be62-b6a05c47b64d")
    def test_list_metadef_objects_in_namespace(self):
        """List Metadef Object Namespace Test

        RBAC test for the glance get_metadef_objects policy
        """
        namespace = self.create_namespace()
        with self.override_role():
            # list md objects
            self.namespace_objects_client.list_namespace_objects(
                namespace['namespace'])

    @rbac_rule_validation.action(service="glance",
                                 rules=["modify_metadef_object"])
    @decorators.idempotent_id("cd130b1d-89fa-479c-a90e-498d895fb455")
    def test_update_metadef_object_in_namespace(self):
        """Update Metadef Object Namespace Test

        RBAC test for the glance modify_metadef_object policy
        """
        namespace = self.create_namespace()
        object_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-object')
        self.namespace_objects_client.create_namespace_object(
            namespace['namespace'],
            name=object_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_objects_client.delete_namespace_object,
                        namespace['namespace'], object_name)

        # Toggle role and modify object
        new_name = "Object New Name"
        with self.override_role():
            self.namespace_objects_client.update_namespace_object(
                namespace['namespace'], object_name, name=new_name)

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_metadef_object"])
    @decorators.idempotent_id("93c61420-5b80-4a0e-b6f3-4ccc6e90b865")
    def test_show_metadef_object_in_namespace(self):
        """Show Metadef Object Namespace Test

        RBAC test for the glance get_metadef_object policy
        """
        namespace = self.create_namespace()
        object_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-object')
        self.namespace_objects_client.create_namespace_object(
            namespace['namespace'],
            name=object_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_objects_client.delete_namespace_object,
                        namespace['namespace'], object_name)
        # Toggle role and get object
        with self.override_role():
            self.namespace_objects_client.show_namespace_object(
                namespace['namespace'],
                object_name)



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_log import log as logging
import testtools

from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF
LOG = logging.getLogger(__name__)


# FIXME(felipemonteiro): `@decorators.attr(type='slow')` are added to tests
# below to in effect cause the tests to be non-voting in Zuul due to a high
# rate of spurious failures related to volume attachments. This will be
# revisited at a later date.
class ServerVolumeAttachmentRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerVolumeAttachmentRbacTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServerVolumeAttachmentRbacTest, cls).setup_clients()
        cls.volumes_client = cls.os_primary.volumes_client_latest

    @classmethod
    def skip_checks(cls):
        super(ServerVolumeAttachmentRbacTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ServerVolumeAttachmentRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')
        cls.volume = cls.create_volume()

    def _recreate_volume(self):
        try:
            # In case detachment failed, update the DB status of the volume
            # to avoid error getting thrown when deleting the volume.
            self.volumes_client.reset_volume_status(
                self.volume['id'], status='available',
                attach_status='detached')
            waiters.wait_for_volume_resource_status(
                self.volumes_client, self.volume['id'], 'available')
            # Next, forcibly delete the volume.
            self.volumes_client.force_delete_volume(self.volume['id'])
            self.volumes_client.wait_for_resource_deletion(self.volume['id'])
        except lib_exc.TimeoutException:
            LOG.exception('Failed to delete volume %s', self.volume['id'])
        # Finally, re-create the volume.
        self.__class__.volume = self.create_volume()

    def _restore_volume_status(self):
        # Forcibly detach any attachments still attached to the volume.
        try:
            attachments = self.volumes_client.show_volume(
                self.volume['id'])['volume']['attachments']
            if attachments:
                # Tests below only ever create one attachment for the volume.
                attachment = attachments[0]
                self.volumes_client.force_detach_volume(
                    self.volume['id'], connector=None,
                    attachment_id=attachment['id'])
                waiters.wait_for_volume_resource_status(self.volumes_client,
                                                        self.volume['id'],
                                                        'available')
        except lib_exc.TimeoutException:
            # If all else fails, rebuild the volume.
            self._recreate_volume()

    def setUp(self):
        super(ServerVolumeAttachmentRbacTest, self).setUp()
        self._restore_volume_status()

    def wait_for_server_volume_swap(self, server_id, old_volume_id,
                                    new_volume_id):
        """Waits for a server to swap the old volume to a new one."""
        volume_attachments = self.servers_client.list_volume_attachments(
            server_id)['volumeAttachments']
        attached_volume_ids = [attachment['volumeId']
                               for attachment in volume_attachments]
        start = int(time.time())

        while (old_volume_id in attached_volume_ids) \
                or (new_volume_id not in attached_volume_ids):
            time.sleep(self.servers_client.build_interval)
            volume_attachments = self.servers_client.list_volume_attachments(
                server_id)['volumeAttachments']
            attached_volume_ids = [attachment['volumeId']
                                   for attachment in volume_attachments]

            if int(time.time()) - start >= self.servers_client.build_timeout:
                old_vol_bdm_status = 'in BDM' \
                    if old_volume_id in attached_volume_ids else 'not in BDM'
                new_vol_bdm_status = 'in BDM' \
                    if new_volume_id in attached_volume_ids else 'not in BDM'
                message = ('Failed to swap old volume %(old_volume_id)s '
                           '(current %(old_vol_bdm_status)s) to new volume '
                           '%(new_volume_id)s (current %(new_vol_bdm_status)s)'
                           ' on server %(server_id)s within the required time '
                           '(%(timeout)s s)' %
                           {'old_volume_id': old_volume_id,
                            'old_vol_bdm_status': old_vol_bdm_status,
                            'new_volume_id': new_volume_id,
                            'new_vol_bdm_status': new_vol_bdm_status,
                            'server_id': server_id,
                            'timeout': self.servers_client.build_timeout})
                raise lib_exc.TimeoutException(message)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-volumes-attachments:index"])
    @decorators.idempotent_id('529b668b-6edb-41d5-8886-d7dbd0614678')
    def test_list_volume_attachments(self):
        with self.override_role():
            self.servers_client.list_volume_attachments(self.server['id'])

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-volumes-attachments:create"])
    @decorators.idempotent_id('21c2c3fd-fbe8-41b1-8ef8-115ec47d54c1')
    def test_create_volume_attachment(self):
        with self.override_role():
            self.servers_client.attach_volume(self.server['id'],
                                              volumeId=self.volume['id'])
        # On teardown detach the volume and wait for it to be available. This
        # is so we don't error out when trying to delete the volume during
        # teardown.
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, self.volume['id'], 'available')
        # Ignore 404s on detach in case the server is deleted or the volume
        # is already detached.
        self.addCleanup(self._detach_volume, self.server, self.volume)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'in-use')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-volumes-attachments:show"])
    @decorators.idempotent_id('997df9c2-6e54-47b6-ab74-e4fdb500f385')
    def test_show_volume_attachment(self):
        attachment = self.attach_volume(self.server, self.volume)

        with self.override_role():
            self.servers_client.show_volume_attachment(
                self.server['id'], attachment['id'])

    @decorators.attr(type='slow')
    @testtools.skipUnless(CONF.compute_feature_enabled.swap_volume,
                          'In-place swapping of volumes not supported.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-volumes-attachments:update"])
    @decorators.idempotent_id('bd667186-eca6-4b78-ab6a-3e2fabcb971f')
    def test_update_volume_attachment(self):
        volume1 = self.volume
        volume2 = self.create_volume()
        # Attach "volume1" to server
        self.attach_volume(self.server, volume1)

        with self.override_role():
            # Swap volume from "volume1" to "volume2"
            self.servers_client.update_attached_volume(
                self.server['id'], volume1['id'], volumeId=volume2['id'])
        self.addCleanup(self._detach_volume, self.server, volume2)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume1['id'], 'available')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume2['id'], 'in-use')
        self.wait_for_server_volume_swap(self.server['id'], volume1['id'],
                                         volume2['id'])

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-volumes-attachments:delete"])
    @decorators.idempotent_id('12b03e90-d087-46af-9c4d-507d021c4984')
    def test_delete_volume_attachment(self):
        self.attach_volume(self.server, self.volume)

        with self.override_role():
            self.servers_client.detach_volume(self.server['id'],
                                              self.volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base

CONF = config.CONF


@testtools.skipIf(
    CONF.policy_feature_enabled.removed_keystone_policies_stein,
    "This policy is unavailable in Stein so cannot be tested.")
class IdentityCredentialsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    def _create_user_project_and_credential(self):
        project = self.setup_test_project()
        user = self.setup_test_user(project_id=project['id'])
        credential = self.setup_test_credential(user=user)
        return credential

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_credential"])
    @decorators.idempotent_id('c1ab6d34-c59f-4ae1-bae9-bb3c1089b48e')
    def test_create_credential(self):
        project = self.setup_test_project()
        user = self.setup_test_user(project_id=project['id'])
        with self.override_role():
            self.setup_test_credential(user=user)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_credential"])
    @decorators.idempotent_id('cfb05ce3-bffb-496e-a3c2-9515d730da63')
    def test_update_credential(self):
        credential = self._create_user_project_and_credential()
        new_keys = [data_utils.rand_uuid_hex(),
                    data_utils.rand_uuid_hex()]

        with self.override_role():
            self.creds_client.update_credential(
                credential['id'],
                credential=credential,
                access_key=new_keys[0],
                secret_key=new_keys[1],
                project_id=credential['project_id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_credential"])
    @decorators.idempotent_id('87ab42af-8d41-401b-90df-21e72919fcde')
    def test_delete_credential(self):
        credential = self._create_user_project_and_credential()

        with self.override_role():
            self.creds_client.delete_credential(credential['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_credential"])
    @decorators.idempotent_id('1b6eeae6-f1e8-4cdf-8903-1c002b1fc271')
    def test_show_credential(self):
        credential = self._create_user_project_and_credential()

        with self.override_role():
            self.creds_client.show_credential(credential['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_credentials"])
    @decorators.idempotent_id('3de303e2-12a7-4811-805a-f18906472038')
    def test_list_credentials(self):
        with self.override_role():
            self.creds_client.list_credentials()



#    Copyright 2017 NEC Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class FixedIpsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    # Tests will fail with a 404 starting from microversion 2.36:
    # See the following link for details:
    # https://developer.openstack.org/api-ref/compute/#fixed-ips-os-fixed-ips-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(FixedIpsRbacTest, cls).skip_checks()
        if CONF.service_available.neutron:
            msg = ("%s skipped as neutron is available" % cls.__name__)
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(FixedIpsRbacTest, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        server = cls.servers_client.show_server(server['id'])['server']
        cls.ip = None
        for ip_set in server['addresses']:
            for ip in server['addresses'][ip_set]:
                if ip['OS-EXT-IPS:type'] == 'fixed':
                    cls.ip = ip['addr']
                    break
            if cls.ip:
                break
        if cls.ip is None:
            raise cls.skipException("No fixed ip found for server: %s"
                                    % server['id'])

    @decorators.idempotent_id('c89391f7-4844-4a70-a116-37c1336efb99')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-fixed-ips"])
    def test_show_fixed_ip_details(self):
        with self.override_role():
            self.fixed_ips_client.show_fixed_ip(self.ip)

    @decorators.idempotent_id('f0314501-735d-4315-9856-959e01e82f0d')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-fixed-ips"])
    def test_set_reserve(self):
        with self.override_role():
            self.fixed_ips_client.reserve_fixed_ip(self.ip, reserve="None")

    @decorators.idempotent_id('866a6fdc-a237-4502-9bf2-52fe82aba356')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-fixed-ips"])
    def test_set_unreserve(self):
        with self.override_role():
            self.fixed_ips_client.reserve_fixed_ip(self.ip, unreserve="None")



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base

CONF = config.CONF


class ProjectTagsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def skip_checks(cls):
        super(ProjectTagsV3RbacTest, cls).skip_checks()
        if not CONF.identity_feature_enabled.project_tags:
            raise cls.skipException("Project tags feature disabled")

    @classmethod
    def resource_setup(cls):
        super(ProjectTagsV3RbacTest, cls).resource_setup()
        cls.project_id = cls.setup_test_project()['id']

    def tearDown(self):
        self.project_tags_client.delete_all_project_tags(self.project_id)
        super(ProjectTagsV3RbacTest, self).tearDown()

    @decorators.idempotent_id('acbd7b2d-0a4d-4990-9fab-eccad69d4238')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_project_tag"])
    def test_update_project_tag(self):
        tag = data_utils.rand_name(self.__class__.__name__ + '-Tag')
        with self.override_role():
            self.project_tags_client.update_project_tag(self.project_id, tag)

    @decorators.idempotent_id('e122d7d1-bb6d-43af-b489-afa8c609b9ae')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_project_tags"])
    def test_list_project_tags(self):
        with self.override_role():
            self.project_tags_client.list_project_tags(self.project_id)

    @decorators.idempotent_id('716f9081-4626-4594-a82c-e7dc037464ac')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_project_tags"])
    def test_update_all_project_tags(self):
        tags = [
            data_utils.rand_name(self.__class__.__name__ + '-Tag')
            for _ in range(2)
        ]
        with self.override_role():
            self.project_tags_client.update_all_project_tags(
                self.project_id, tags)

    @decorators.idempotent_id('974cb1da-d7d4-4863-99da-4a3f0c801729')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_project_tag"])
    def test_check_project_tag_existence(self):
        tag = data_utils.rand_name(self.__class__.__name__ + '-Tag')
        self.project_tags_client.update_project_tag(self.project_id, tag)

        with self.override_role():
            self.project_tags_client.check_project_tag_existence(
                self.project_id, tag)

    @decorators.idempotent_id('ffe0c8e1-f9eb-43c5-8097-1e938fc08e07')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_project_tag"])
    def test_delete_project_tag(self):
        tag = data_utils.rand_name(self.__class__.__name__ + '-Tag')
        self.project_tags_client.update_project_tag(self.project_id, tag)

        with self.override_role():
            self.project_tags_client.delete_project_tag(self.project_id, tag)

    @decorators.idempotent_id('94d0ef63-e9e3-4287-9c5e-bd5464467d77')
    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_project_tags"])
    def test_delete_all_project_tags(self):
        with self.override_role():
            self.project_tags_client.delete_all_project_tags(self.project_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class AgentsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AgentsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('agent', 'network'):
            msg = "agent extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(AgentsRbacTest, cls).resource_setup()
        agents = cls.agents_client.list_agents()['agents']
        cls.agent = agents[0]

    @decorators.idempotent_id('f88e38e0-ab52-4b97-8ffa-48a27f9d199b')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_agent"],
                                 expected_error_codes=[404])
    def test_show_agent(self):
        """Show agent test.

        RBAC test for the neutron get_agent policy
        """
        with self.override_role():
            self.agents_client.show_agent(self.agent['id'])

    @decorators.idempotent_id('8ca68fdb-eaf6-4880-af82-ba0982949dec')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_agent", "update_agent"],
                                 expected_error_codes=[404, 403])
    def test_update_agent(self):
        """Update agent test.

        RBAC test for the neutron update_agent policy
        """
        original_status = self.agent['admin_state_up']
        agent_status = {'admin_state_up': original_status}

        with self.override_role():
            self.agents_client.update_agent(agent_id=self.agent['id'],
                                            agent=agent_status)

    @decorators.idempotent_id('f7a085e2-71b1-4d39-be3e-fea4bc10ccb8')
    @rbac_rule_validation.action(service="neutron", rules=["get_agent"])
    def test_list_agents(self):
        """List agents test.

        RBAC test for the neutron ``list_agents`` function and
        the ``get_agent`` policy
        """
        admin_resource_id = self.agent['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.agents_client.list_agents(
                id=admin_resource_id)["agents"]


class L3AgentSchedulerRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(L3AgentSchedulerRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('l3_agent_scheduler', 'network'):
            msg = "l3_agent_scheduler extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(L3AgentSchedulerRbacTest, cls).resource_setup()
        cls.router = cls.create_router()
        cls.agent = None

    def setUp(self):
        super(L3AgentSchedulerRbacTest, self).setUp()
        if self.agent is not None:
            return

        # Find an agent and validate that it is correct.
        agents = self.agents_client.list_agents()['agents']
        agent = {'agent_type': None}
        for a in agents:
            if a['agent_type'] == 'L3 agent':
                agent = a
                break
        self.assertEqual(agent['agent_type'], 'L3 agent', 'Could not find '
                         'L3 agent in agent list though l3_agent_scheduler '
                         'is enabled.')
        self.agent = agent

    @decorators.idempotent_id('5d2bbdbc-40a5-43d2-828a-84dc93fcc453')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_l3-routers"])
    def test_list_routers_on_l3_agent(self):
        """List routers on L3 agent test.

        RBAC test for the neutron get_l3-routers policy
        """
        with self.override_role():
            self.agents_client.list_routers_on_l3_agent(self.agent['id'])

    @decorators.idempotent_id('466b2a10-8747-4c09-855a-bd90a1c86ce7')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_l3-router"])
    def test_create_router_on_l3_agent(self):
        """Create router on L3 agent test.

        RBAC test for the neutron create_l3-router policy
        """
        with self.override_role():
            self.agents_client.create_router_on_l3_agent(
                self.agent['id'], router_id=self.router['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.agents_client.delete_router_from_l3_agent,
            self.agent['id'], router_id=self.router['id'])

    @decorators.idempotent_id('8138cfc9-3e48-4a34-adf6-894077aa1be4')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["delete_l3-router"])
    def test_delete_router_from_l3_agent(self):
        """Delete router from L3 agent test.

        RBAC test for the neutron delete_l3-router policy
        """
        self.agents_client.create_router_on_l3_agent(
            self.agent['id'], router_id=self.router['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.agents_client.delete_router_from_l3_agent,
            self.agent['id'], router_id=self.router['id'])

        with self.override_role():
            self.agents_client.delete_router_from_l3_agent(
                self.agent['id'], router_id=self.router['id'])


class DHCPAgentSchedulersRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(DHCPAgentSchedulersRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('dhcp_agent_scheduler', 'network'):
            msg = "dhcp_agent_scheduler extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(DHCPAgentSchedulersRbacTest, cls).resource_setup()
        cls.agent = None

    def setUp(self):
        super(DHCPAgentSchedulersRbacTest, self).setUp()
        if self.agent is not None:
            return

        # Find a DHCP agent and validate that it is correct.
        agents = self.agents_client.list_agents()['agents']
        agent = {'agent_type': None}
        for a in agents:
            if a['agent_type'] == 'DHCP agent':
                agent = a
                break
        self.assertEqual(agent['agent_type'], 'DHCP agent', 'Could not find '
                         'DHCP agent in agent list though dhcp_agent_scheduler'
                         ' is enabled.')
        self.agent = agent

    def _create_and_prepare_network_for_agent(self, agent_id):
        """Create network and ensure it is not hosted by agent_id."""
        network_id = self.create_network()['id']

        if self._check_network_in_dhcp_agent(network_id, agent_id):
            self.agents_client.delete_network_from_dhcp_agent(
                agent_id=agent_id, network_id=network_id)

        return network_id

    def _check_network_in_dhcp_agent(self, network_id, agent_id):
        networks = self.agents_client.list_networks_hosted_by_one_dhcp_agent(
            agent_id)['networks'] or []
        return network_id in [network['id'] for network in networks]

    @decorators.idempotent_id('dc84087b-4c2a-4878-8ed0-40370e19da17')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_dhcp-networks"])
    def test_list_networks_hosted_by_one_dhcp_agent(self):
        """List networks hosted by one DHCP agent test.

        RBAC test for the neutron get_dhcp-networks policy
        """
        with self.override_role():
            self.agents_client.list_networks_hosted_by_one_dhcp_agent(
                self.agent['id'])

    @decorators.idempotent_id('14e014ac-f355-46d3-b6d8-98f2c9ec1610')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_dhcp-network"])
    def test_add_dhcp_agent_to_network(self):
        """Add DHCP agent to network test.

        RBAC test for the neutron create_dhcp-network policy
        """
        network_id = self._create_and_prepare_network_for_agent(
            self.agent['id'])

        with self.override_role():
            self.agents_client.add_dhcp_agent_to_network(
                self.agent['id'], network_id=network_id)
        # Clean up is not necessary and might result in 409 being raised.

    @decorators.idempotent_id('937a4302-4b49-407d-9980-5843d7badc38')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["delete_dhcp-network"])
    def test_delete_network_from_dhcp_agent(self):
        """Delete DHCP agent from network test.

        RBAC test for the neutron delete_dhcp-network policy
        """
        network_id = self._create_and_prepare_network_for_agent(
            self.agent['id'])
        self.agents_client.add_dhcp_agent_to_network(
            self.agent['id'], network_id=network_id)
        # Clean up is not necessary and might result in 409 being raised.

        with self.override_role():
            self.agents_client.delete_network_from_dhcp_agent(
                self.agent['id'], network_id=network_id)


class L3AgentsExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(L3AgentsExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('l3_agent_scheduler', 'network'):
            msg = "l3_agent_scheduler extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(L3AgentsExtRbacTest, cls).resource_setup()
        name = data_utils.rand_name(cls.__name__ + '-Router')
        cls.router = cls.ntp_client.create_router(name)['router']

    @decorators.idempotent_id('5d2bbdbc-40a5-43d2-828a-84dc93bcd321')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_l3-agents"])
    def test_list_l3_agents_on_router(self):
        """List L3 agents on router test.

        RBAC test for the neutron get_l3-agents policy
        """
        with self.override_role():
            # NOTE: It is not empty list since it's a special case where
            # policy.enforce is called from the controller.
            self.ntp_client.list_l3_agents_hosting_router(self.router['id'])



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


CONF = config.CONF


class ApplicationCredentialsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def skip_checks(cls):
        super(ApplicationCredentialsV3RbacTest, cls).skip_checks()
        if not CONF.identity_feature_enabled.application_credentials:
            raise cls.skipException("Application credentials are not available"
                                    " in this environment")

    @classmethod
    def resource_setup(cls):
        super(ApplicationCredentialsV3RbacTest, cls).resource_setup()
        cls.user_id = cls.os_primary.credentials.user_id

    def _create_application_credential(self, name=None, **kwargs):
        name = name or data_utils.rand_name('application_credential')
        application_credential = (
            self.application_credentials_client.create_application_credential(
                self.user_id, name=name, **kwargs))['application_credential']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.application_credentials_client.delete_application_credential,
            self.user_id,
            application_credential['id'])
        return application_credential

    @decorators.idempotent_id('b53bee14-e9df-4929-b257-6def76c12e4d')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:create_application_credential"])
    def test_create_application_credential(self):
        with self.override_role():
            self._create_application_credential()

    @decorators.idempotent_id('58b3c3a0-5ad0-44f7-8da7-0736f71f7168')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:list_application_credentials"])
    def test_list_application_credentials(self):
        with self.override_role():
            self.application_credentials_client.list_application_credentials(
                user_id=self.user_id)

    @decorators.idempotent_id('d7b13968-a8a6-47fd-8e1d-7cc7f565c7f8')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:get_application_credential"])
    def test_show_application_credential(self):
        app_cred = self._create_application_credential()
        with self.override_role():
            self.application_credentials_client.show_application_credential(
                user_id=self.user_id, application_credential_id=app_cred['id'])

    @decorators.idempotent_id('521b7c0f-1dd5-47a6-ae95-95c0323d7735')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:delete_application_credential"])
    def test_delete_application_credential(self):
        app_cred = self._create_application_credential()
        with self.override_role():
            self.application_credentials_client.delete_application_credential(
                user_id=self.user_id, application_credential_id=app_cred['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class SubnetPoolsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(SubnetPoolsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('subnet_allocation', 'network'):
            msg = "subnet_allocation extension not enabled."
            raise cls.skipException(msg)

    def _create_subnetpool(self, **kwargs):
        post_body = {'name': data_utils.rand_name(self.__class__.__name__),
                     'min_prefixlen': 24,
                     'max_prefixlen': 32,
                     'prefixes': [CONF.network.project_network_cidr]}

        if kwargs:
            post_body.update(kwargs)

        body = self.subnetpools_client.create_subnetpool(**post_body)
        subnetpool = body['subnetpool']

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnetpools_client.delete_subnetpool,
                        subnetpool['id'])

        return subnetpool

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_subnetpool"])
    @decorators.idempotent_id('1b5509fd-2c32-44a8-a786-1b6ca162dbd1')
    def test_create_subnetpool(self):
        """Create subnetpool.

        RBAC test for the neutron create_subnetpool policy
        """
        with self.override_role():
            self._create_subnetpool()

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_subnetpool",
                                        "create_subnetpool:is_default"],
                                 expected_error_codes=[403, 403])
    @decorators.idempotent_id('1b5509fd-2c32-44a8-a786-1b6ca162dbd2')
    def test_create_subnetpool_default(self):
        """Create default subnetpool.

        RBAC test for the neutron create_subnetpool:is_default policy
        """
        # Most likely we already have default subnetpools for ipv4 and ipv6,
        # so we temporary mark them as is_default=False, to let this test pass.
        def_pools = self.subnetpools_client.list_subnetpools(is_default=True)
        for default_pool in def_pools["subnetpools"]:
            self.subnetpools_client.update_subnetpool(default_pool["id"],
                                                      is_default=False)

            self.addCleanup(self.subnetpools_client.update_subnetpool,
                            default_pool["id"], is_default=True)

        with self.override_role():
            # It apparently only enforces the policy for is_default=True.
            # It does nothing for is_default=False
            self._create_subnetpool(is_default=True)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_subnetpool",
                                        "create_subnetpool:shared"],
                                 expected_error_codes=[403, 403])
    @decorators.idempotent_id('cf730989-0d47-40bc-b39a-99e7de484723')
    def test_create_subnetpool_shared(self):
        """Create subnetpool shared.

        RBAC test for the neutron create_subnetpool:shared policy
        """
        with self.override_role():
            self._create_subnetpool(shared=True)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_subnetpool"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('4f5aee26-0507-4b6d-b44c-3128a25094d2')
    def test_show_subnetpool(self):
        """Show subnetpool.

        RBAC test for the neutron get_subnetpool policy
        """
        subnetpool = self._create_subnetpool()
        with self.override_role():
            self.subnetpools_client.show_subnetpool(subnetpool['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_subnetpool",
                                        "update_subnetpool"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('1e79cead-5081-4be2-a4f7-484c0f443b9b')
    def test_update_subnetpool(self):
        """Update subnetpool.

        RBAC test for the neutron update_subnetpool policy
        """
        subnetpool = self._create_subnetpool()
        with self.override_role():
            self.subnetpools_client.update_subnetpool(subnetpool['id'],
                                                      min_prefixlen=24)

    @decorators.idempotent_id('a16f4e5c-0675-415f-b636-00af00638693')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["update_subnetpool",
                                        "update_subnetpool:is_default"])
    def test_update_subnetpool_is_default(self):
        """Update default subnetpool.

        RBAC test for the neutron update_subnetpool:is_default policy
        """
        subnetpools = self.subnetpools_client.list_subnetpools()['subnetpools']
        default_pool = list(
            filter(lambda p: p['is_default'] is True, subnetpools))
        if default_pool:
            default_pool = default_pool[0]
        else:
            default_pool = self._create_subnetpool(is_default=True)
        original_desc = default_pool['description']

        with self.override_role():
            self.subnetpools_client.update_subnetpool(
                default_pool['id'], description=original_desc, is_default=True)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_subnetpool",
                                        "delete_subnetpool"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('50f5944e-43e5-457b-ab50-fb48a73f0d3e')
    def test_delete_subnetpool(self):
        """Delete subnetpool.

        RBAC test for the neutron delete_subnetpool policy
        """
        subnetpool = self._create_subnetpool()
        with self.override_role():
            self.subnetpools_client.delete_subnetpool(subnetpool['id'])

    @rbac_rule_validation.action(service="neutron", rules=["get_subnetpool"])
    @decorators.idempotent_id('f1caf0f6-bde5-11e8-a355-529269fb1459')
    def test_list_subnetpools(self):
        """List subnetpools.

        RBAC test for the neutron ``list_subnetpools`` function and
        the ``get_subnetpool`` policy
        """
        admin_resource_id = self._create_subnetpool()['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.subnetpools_client.list_subnetpools(
                id=admin_resource_id)["subnetpools"]



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumesBasicCrudV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def resource_setup(cls):
        super(VolumesBasicCrudV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:create"])
    @decorators.idempotent_id('426b08ef-6394-4d06-9128-965d5a6c38ef')
    def test_create_volume(self):
        name = data_utils.rand_name(self.__class__.__name__ + '-Volume')
        size = CONF.volume.volume_size

        with self.override_role():
            volume = self.volumes_client.create_volume(name=name, size=size)[
                'volume']
        # Use helper in base Tempest volume class which waits for deletion.
        self.addCleanup(self.delete_volume, self.volumes_client, volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

    @decorators.idempotent_id('a009e6dc-e8bf-4412-99f5-8e45cffcffec')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:create_from_image"])
    def test_create_volume_from_image(self):
        name = data_utils.rand_name(self.__class__.__name__ + '-Volume')
        size = CONF.volume.volume_size
        img_uuid = CONF.compute.image_ref

        with self.override_role():
            volume = self.volumes_client.create_volume(
                name=name, size=size, imageRef=img_uuid)['volume']
        # Use helper in base Tempest volume class which waits for deletion.
        self.addCleanup(self.delete_volume, self.volumes_client, volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:delete"])
    @decorators.idempotent_id('6de9f9c2-509f-4558-867b-af21c7163be4')
    def test_delete_volume(self):
        volume = self.create_volume()
        with self.override_role():
            self.volumes_client.delete_volume(volume['id'])
        self.volumes_client.wait_for_resource_deletion(volume['id'])

    @rbac_rule_validation.action(service="cinder", rules=["volume:get"])
    @decorators.idempotent_id('c4c3fdd5-b1b1-49c3-b977-a9f40ee9257a')
    def test_show_volume(self):
        with self.override_role():
            self.volumes_client.show_volume(self.volume['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_all"])
    @decorators.idempotent_id('e3ab7906-b04b-4c45-aa11-1104d302f940')
    def test_list_volumes(self):
        with self.override_role():
            self.volumes_client.list_volumes()

    @decorators.idempotent_id('9b6d5beb-254f-4f1b-9906-0bdce4042f53')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_all"])
    def test_list_volumes_with_details(self):
        with self.override_role():
            self.volumes_client.list_volumes(detail=True)

    @rbac_rule_validation.action(service="cinder", rules=["volume:update"])
    @decorators.idempotent_id('b751b889-9a9b-40d8-ae7d-4b0f65e71ac7')
    def test_update_volume(self):
        update_name = data_utils.rand_name(self.__class__.__name__ + 'volume')
        with self.override_role():
            self.volumes_client.update_volume(self.volume['id'],
                                              name=update_name)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.api.network import base as network_base
from tempest.lib.common.utils import test_utils

from patrole_tempest_plugin import rbac_utils


class BaseNetworkRbacTest(rbac_utils.RbacUtilsMixin,
                          network_base.BaseNetworkTest):
    pass


class BaseNetworkExtRbacTest(BaseNetworkRbacTest):
    """Base class to be used with tests that require neutron-tempest-plugin.
    """

    @classmethod
    def get_auth_providers(cls):
        """Register auth_provider from neutron-tempest-plugin.
        """
        providers = super(BaseNetworkExtRbacTest, cls).get_auth_providers()
        if cls.is_neutron_tempest_plugin_available():
            providers.append(cls.ntp_client.auth_provider)
        return providers

    @classmethod
    def skip_checks(cls):
        super(BaseNetworkExtRbacTest, cls).skip_checks()

        if not cls.is_neutron_tempest_plugin_available():
            msg = ("neutron-tempest-plugin not installed.")
            raise cls.skipException(msg)

    @classmethod
    def is_neutron_tempest_plugin_available(cls):
        try:
            import neutron_tempest_plugin  # noqa
            return True
        except ImportError:
            return False

    @classmethod
    def get_client_manager(cls, credential_type=None, roles=None,
                           force_new=None):
        manager = super(BaseNetworkExtRbacTest, cls).get_client_manager(
            credential_type=credential_type,
            roles=roles,
            force_new=force_new
        )

        # Import neutron-tempest-plugin clients
        if cls.is_neutron_tempest_plugin_available():
            from neutron_tempest_plugin.api import clients
            neutron_tempest_manager = clients.Manager(manager.credentials)
            cls.ntp_client = neutron_tempest_manager.network_client

        return manager

    @classmethod
    def create_service_profile(cls):
        service_profile = cls.ntp_client.create_service_profile(
            metainfo=json.dumps({'foo': 'bar'}))
        service_profile_id = service_profile["service_profile"]["id"]
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.ntp_client.delete_service_profile, service_profile_id)
        return service_profile_id



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class TenantNetworksRbacTest(rbac_base.BaseV2ComputeRbacTest):

    # Tests will fail with a 404 starting from microversion 2.36.
    # See the following link for details:
    # https://developer.openstack.org/api-ref/compute/#project-networks-os-tenant-networks-deprecated
    max_microversion = '2.35'

    @classmethod
    def setup_clients(cls):
        super(TenantNetworksRbacTest, cls).setup_clients()
        cls.tenant_networks_client = cls.os_primary.tenant_networks_client

    @classmethod
    def skip_checks(cls):
        super(TenantNetworksRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-tenant-networks', 'compute'):
            msg = "os-tenant-networks extension not enabled."
            raise cls.skipException(msg)
        if not CONF.service_available.neutron:
            raise cls.skipException(
                '%s skipped as Neutron is required' % cls.__name__)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True)
        super(TenantNetworksRbacTest, cls).setup_credentials()

    @decorators.idempotent_id('42b39ba1-14aa-4799-9518-34367d0da67a')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-tenant-networks"])
    def test_list_show_tenant_networks(self):
        with self.override_role():
            self.tenant_networks_client.list_tenant_networks()



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class MiscPolicyActionsRbacTest(rbac_base.BaseV2ComputeRbacTest):
    """Test multiple policy actions that require a server to be created.

    Minimize the number of servers that need to be created across classes
    by consolidating test cases related to different policy "families" into
    one class. This reduces the risk of running into `BuildErrorException`
    errors being raised due to too many servers being created simultaneously
    especially when higher concurrency is used.

    Only applies to:
      * policy "families" that require server creation
      * small policy "families" -- i.e. containing one to three policies

    Tests are ordered by policy name.
    """

    credentials = ['primary', 'admin']

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(MiscPolicyActionsRbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(MiscPolicyActionsRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    def setUp(self):
        super(MiscPolicyActionsRbacTest, self).setUp()
        try:
            waiters.wait_for_server_status(self.servers_client,
                                           self.server['id'], 'ACTIVE')
        except lib_exc.NotFound:
            # If the server was found to be deleted by a previous test,
            # a new one is built
            self.__class__.server = self.create_test_server(
                wait_until='ACTIVE')
        except Exception:
            # Rebuilding the server in case something happened during a test
            self.__class__.server = self._rebuild_server(self.server['id'])

    def _rebuild_server(self, server_id):
        # Destroy an existing server and creates a new one.
        if server_id:
            self.delete_server(server_id)

        self.password = data_utils.rand_password()
        return self.create_test_server(
            wait_until='ACTIVE', adminPass=self.password)

    @utils.requires_ext(extension='os-admin-actions', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-admin-actions:reset_state"])
    @decorators.idempotent_id('ae84dd0b-f364-462e-b565-3457f9c019ef')
    def test_reset_server_state(self):
        """Test reset server state, part of os-admin-actions."""
        with self.override_role():
            self.servers_client.reset_state(self.server['id'], state='error')
        self.addCleanup(self.servers_client.reset_state, self.server['id'],
                        state='active')

    @utils.requires_ext(extension='os-admin-actions', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-admin-actions:inject_network_info"])
    @decorators.idempotent_id('ce48c340-51c1-4cff-9b6e-0cc5ef008630')
    def test_inject_network_info(self):
        """Test inject network info, part of os-admin-actions."""
        with self.override_role():
            self.servers_client.inject_network_info(self.server['id'])

    @utils.requires_ext(extension='os-admin-actions', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-admin-actions:reset_network"])
    @decorators.idempotent_id('2911a242-15c4-4fcb-80d5-80a8930661b0')
    def test_reset_network(self):
        """Test reset network, part of os-admin-actions."""
        with self.override_role():
            self.servers_client.reset_network(self.server['id'])

    @testtools.skipUnless(CONF.compute_feature_enabled.change_password,
                          'Change password not available.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-admin-password"])
    @decorators.idempotent_id('908a7d59-3a66-441c-94cf-38e57ed14956')
    def test_change_server_password(self):
        """Test change admin password, part of os-admin-password."""
        original_password = self.servers_client.show_password(
            self.server['id'])

        with self.override_role():
            self.servers_client.change_password(
                self.server['id'], adminPass=data_utils.rand_password())
        self.addCleanup(self.servers_client.change_password, self.server['id'],
                        adminPass=original_password)
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'ACTIVE')

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @utils.requires_ext(extension='os-config-drive', service='compute')
    @decorators.idempotent_id('2c82e819-382d-4d6f-87f0-a45954cbbc64')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-config-drive"])
    def test_list_servers_with_details_config_drive(self):
        """Test list servers with config_drive property in response body."""
        with self.override_role():
            body = self.servers_client.list_servers(detail=True)['servers']
        expected_attr = 'config_drive'
        # If the first server contains "config_drive", then all the others do.
        if expected_attr not in body[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @utils.requires_ext(extension='os-config-drive', service='compute')
    @decorators.idempotent_id('55c62ef7-b72b-4970-acc6-05b0a4316e5d')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-config-drive"])
    def test_show_server_config_drive(self):
        """Test show server with config_drive property in response body."""
        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']
        expected_attr = 'config_drive'
        if expected_attr not in body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @utils.requires_ext(extension='os-deferred-delete', service='compute')
    @decorators.idempotent_id('189bfed4-1e6d-475c-bb8c-d57e60895391')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-deferred-delete"])
    def test_force_delete_server(self):
        """Test force delete server, part of os-deferred-delete."""
        with self.override_role():
            # Force-deleting a server enforces os-deferred-delete.
            self.servers_client.force_delete_server(self.server['id'])

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('d873740a-7b10-40a9-943d-7cc18115370e')
    @utils.requires_ext(extension='OS-EXT-AZ', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-availability-zone"])
    def test_list_servers_with_details_extended_availability_zone(self):
        """Test list servers OS-EXT-AZ:availability_zone attr in resp body."""
        expected_attr = 'OS-EXT-AZ:availability_zone'

        with self.override_role():
            body = self.servers_client.list_servers(detail=True)['servers']
        # If the first server contains `expected_attr`, then all the others do.
        if expected_attr not in body[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('727e5360-770a-4b9c-8015-513a40216635')
    @utils.requires_ext(extension='OS-EXT-AZ', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-availability-zone"])
    def test_show_server_extended_availability_zone(self):
        """Test show server OS-EXT-AZ:availability_zone attr in resp body."""
        expected_attr = 'OS-EXT-AZ:availability_zone'

        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']
        if expected_attr not in body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('4aa5d93e-4887-468a-8eb4-b6eca0ca6437')
    @utils.requires_ext(extension='OS-EXT-SRV-ATTR', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-server-attributes"])
    def test_list_servers_extended_server_attributes(self):
        """Test list servers with details, with extended server attributes in
        response body.
        """
        with self.override_role():
            body = self.servers_client.list_servers(detail=True)['servers']

        # NOTE(felipemonteiro): The attributes included below should be
        # returned by all microversions. We don't include tests for other
        # microversions since Tempest schema validation takes care of that in
        # `show_server` call above. (Attributes there are *optional*.)
        for attr in ('host', 'instance_name'):
            whole_attr = 'OS-EXT-SRV-ATTR:%s' % attr
            if whole_attr not in body[0]:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=whole_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('2ed7aee2-94b2-4a9f-ae63-a51b7f94fe30')
    @utils.requires_ext(extension='OS-EXT-SRV-ATTR', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-server-attributes"])
    def test_show_server_extended_server_attributes(self):
        """Test show server with extended server attributes in response
        body.
        """
        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']

        # NOTE(felipemonteiro): The attributes included below should be
        # returned by all microversions. We don't include tests for other
        # microversions since Tempest schema validation takes care of that in
        # `show_server` call above. (Attributes there are *optional*.)
        for attr in ('host', 'instance_name'):
            whole_attr = 'OS-EXT-SRV-ATTR:%s' % attr
            if whole_attr not in body:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=whole_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('82053c27-3134-4003-9b55-bc9fafdb0e3b')
    @utils.requires_ext(extension='OS-EXT-STS', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-status"])
    def test_list_servers_extended_status(self):
        """Test list servers with extended properties in response body."""
        with self.override_role():
            body = self.servers_client.list_servers(detail=True)['servers']

        expected_attrs = ('OS-EXT-STS:task_state', 'OS-EXT-STS:vm_state',
                          'OS-EXT-STS:power_state')
        for attr in expected_attrs:
            if attr not in body[0]:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('7d2620a5-eea1-4a8b-96ea-86ad77a73fc8')
    @utils.requires_ext(extension='OS-EXT-STS', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-status"])
    def test_show_server_extended_status(self):
        """Test show server with extended properties in response body."""
        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']

        expected_attrs = ('OS-EXT-STS:task_state', 'OS-EXT-STS:vm_state',
                          'OS-EXT-STS:power_state')
        for attr in expected_attrs:
            if attr not in body:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('21e39cbe-6c32-48fc-80dd-3e1fece6053f')
    @utils.requires_ext(extension='os-extended-volumes', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-volumes"])
    def test_list_servers_with_details_extended_volumes(self):
        """Test list servers os-extended-volumes:volumes_attached attr in resp
        body.
        """
        expected_attr = 'os-extended-volumes:volumes_attached'

        with self.override_role():
            body = self.servers_client.list_servers(detail=True)['servers']
        if expected_attr not in body[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @decorators.idempotent_id('7f163708-0d25-4138-8512-dfdd72a92989')
    @utils.requires_ext(extension='os-extended-volumes', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-extended-volumes"])
    def test_show_server_extended_volumes(self):
        """Test show server os-extended-volumes:volumes_attached attr in resp
        body.
        """
        expected_attr = 'os-extended-volumes:volumes_attached'

        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']
        if expected_attr not in body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @utils.requires_ext(extension='os-instance-actions', service='compute')
    @decorators.idempotent_id('9d1b131d-407e-4fa3-8eef-eb2c4526f1da')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-instance-actions"])
    def test_list_instance_actions(self):
        """Test list instance actions, part of os-instance-actions."""
        with self.override_role():
            self.servers_client.list_instance_actions(self.server['id'])

    @utils.requires_ext(extension='os-instance-actions', service='compute')
    @decorators.idempotent_id('eb04c439-4215-4029-9ccb-5b3c041bfc25')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-instance-actions:events"])
    def test_show_instance_action(self):
        """Test show instance action, part of os-instance-actions.

        Expect "events" details to be included in the response body.
        """
        # NOTE: "os_compute_api:os-instance-actions" is also enforced.
        request_id = self.server.response['x-compute-request-id']

        with self.override_role():
            instance_action = self.servers_client.show_instance_action(
                self.server['id'], request_id)['instanceAction']

        if 'events' not in instance_action:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute='events')
        # Microversion 2.51+ returns 'events' always, but not 'traceback'. If
        # 'traceback' is also present then policy enforcement passed.
        if 'traceback' not in instance_action['events'][0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute='events.traceback')

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs"])
    @decorators.idempotent_id('81e6fa34-c06b-42ca-b195-82bf8699b940')
    def test_show_server_keypair(self):
        with self.override_role():
            result = self.servers_client.show_server(self.server['id'])[
                'server']
        if 'key_name' not in result:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute='key_name')

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs"])
    @decorators.idempotent_id('41ca4280-ec59-4b80-a9b1-6bc6366faf39')
    def test_list_servers_keypairs(self):
        with self.override_role():
            result = self.servers_client.list_servers(detail=True)['servers']
        if 'key_name' not in result[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute='key_name')

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-lock-server:lock"])
    @decorators.idempotent_id('b81e10fb-1864-498f-8c1d-5175c6fec5fb')
    def test_lock_server(self):
        """Test lock server, part of os-lock-server."""
        with self.override_role():
            self.servers_client.lock_server(self.server['id'])
        self.addCleanup(self.servers_client.unlock_server, self.server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-lock-server:unlock"])
    @decorators.idempotent_id('d50ef8e8-4bce-11e7-b114-b2f933d5fe66')
    def test_unlock_server(self):
        """Test unlock server, part of os-lock-server."""
        self.servers_client.lock_server(self.server['id'])
        self.addCleanup(self.servers_client.unlock_server, self.server['id'])

        with self.override_role():
            self.servers_client.unlock_server(self.server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-lock-server:unlock",
               "os_compute_api:os-lock-server:unlock:unlock_override"])
    @decorators.idempotent_id('40dfeef9-73ee-48a9-be19-a219875de457')
    def test_unlock_server_override(self):
        """Test force unlock server, part of os-lock-server.

        In order to trigger the unlock:unlock_override policy instead
        of the unlock policy, the server must be locked by a different
        user than the one who is attempting to unlock it.
        """
        self.os_admin.servers_client.lock_server(self.server['id'])
        self.addCleanup(self.servers_client.unlock_server, self.server['id'])

        with self.override_role():
            self.servers_client.unlock_server(self.server['id'])

    @utils.requires_ext(extension='os-rescue', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-rescue"])
    @decorators.idempotent_id('fbbb2afc-ed0e-4552-887d-ac00fb5d436e')
    def test_rescue_server(self):
        """Test rescue server, part of os-rescue."""
        with self.override_role():
            self.servers_client.rescue_server(self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'RESCUE')

    @decorators.idempotent_id('ac2d956f-d6a3-4184-b814-b44d05c9574c')
    @utils.requires_ext(extension='os-rescue', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-rescue"])
    def test_unrescue_server(self):
        """Test unrescue server, part of os-rescue."""
        self.servers_client.rescue_server(self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'RESCUE')

        with self.override_role():
            self.servers_client.unrescue_server(self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'ACTIVE')

    @utils.requires_ext(extension='os-server-diagnostics', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-diagnostics"])
    @decorators.idempotent_id('5dabfcc4-bedb-417b-8247-b3ee7c5c0f3e')
    def test_show_server_diagnostics(self):
        """Test show server diagnostics, part of os-server-diagnostics."""
        with self.override_role():
            self.servers_client.show_server_diagnostics(self.server['id'])

    @utils.requires_ext(extension='os-server-password', service='compute')
    @decorators.idempotent_id('aaf43f78-c178-4581-ac18-14afd3f1f6ba')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-password"])
    def test_delete_server_password(self):
        """Test delete server password, part of os-server-password."""
        with self.override_role():
            self.servers_client.delete_password(self.server['id'])

    @utils.requires_ext(extension='os-server-password', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-password"])
    @decorators.idempotent_id('f677971a-7d20-493c-977f-6ff0a74b5b2c')
    def test_get_server_password(self):
        """Test show server password, part of os-server-password."""
        with self.override_role():
            self.servers_client.show_password(self.server['id'])

    @testtools.skipIf(CONF.policy_feature_enabled.removed_nova_policies_stein,
                      "This API extension policy was removed in Stein")
    @utils.requires_ext(extension='OS-SRV-USG', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-server-usage"])
    @decorators.idempotent_id('f0437ead-b9fb-462a-9f3d-ce53fac9d57a')
    def test_show_server_usage(self):
        """Test show server usage, part of os-server-usage.

        TODO(felipemonteiro): Once multiple policy testing is supported, this
        test should also check for additional policies mentioned here:
        https://git.openstack.org/cgit/openstack/nova/tree/nova/policies/server_usage.py?h=17.0.0
        """
        expected_attrs = ('OS-SRV-USG:launched_at',
                          'OS-SRV-USG:terminated_at')

        with self.override_role():
            body = self.servers_client.show_server(self.server['id'])['server']
        for expected_attr in expected_attrs:
            if expected_attr not in body:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=expected_attr)

    @utils.requires_ext(extension='os-simple-tenant-usage', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-simple-tenant-usage:list"])
    @decorators.idempotent_id('2aef094f-0452-4df6-a66a-0ec22a92b16e')
    def test_list_simple_tenant_usages(self):
        """Test list tenant usages, part of os-simple-tenant-usage."""
        with self.override_role():
            self.tenant_usages_client.list_tenant_usages()

    @utils.requires_ext(extension='os-simple-tenant-usage', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-simple-tenant-usage:show"])
    @decorators.idempotent_id('fe7eacda-15c4-4bf7-93ef-1091c4546a9d')
    def test_show_simple_tenant_usage(self):
        """Test show tenant usage, part of os-simple-tenant-usage."""
        tenant_id = self.os_primary.credentials.tenant_id

        with self.override_role():
            self.tenant_usages_client.show_tenant_usage(tenant_id=tenant_id)

    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          "Suspend compute feature is not available.")
    @decorators.idempotent_id('b775930f-237c-431c-83ae-d33ed1b9700b')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-suspend-server:suspend"])
    def test_suspend_server(self):
        """Test suspend server, part of os-suspend-server."""
        with self.override_role():
            self.servers_client.suspend_server(self.server['id'])
        self.addCleanup(self.servers_client.resume_server, self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'SUSPENDED')

    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          "Suspend compute feature is not available.")
    @decorators.idempotent_id('4d90bd02-11f8-45b1-a8a1-534665584675')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-suspend-server:resume"])
    def test_resume_server(self):
        """Test resume server, part of os-suspend-server."""
        self.servers_client.suspend_server(self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'SUSPENDED')

        with self.override_role():
            self.servers_client.resume_server(self.server['id'])
        waiters.wait_for_server_status(
            self.servers_client, self.server['id'], 'ACTIVE')


class MiscPolicyActionsNetworkRbacTest(rbac_base.BaseV2ComputeRbacTest):
    """Test multiple policy actions that require a server to be created.

    Only applies to:
      * policy "families" that require server creation
      * small policy "families" -- i.e. containing one to three policies
      * tests that require network resources
    """

    @classmethod
    def skip_checks(cls):
        super(MiscPolicyActionsNetworkRbacTest, cls).skip_checks()
        # All tests below require Neutron availability.
        if not CONF.service_available.neutron:
            raise cls.skipException(
                '%s skipped as Neutron is required' % cls.__name__)

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(MiscPolicyActionsNetworkRbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        def _cleanup_ports(network_id):
            ports = cls.ports_client.list_ports(network_id=network_id)['ports']
            for port in ports:
                test_utils.call_and_ignore_notfound_exc(
                    cls.ports_client.delete_port,
                    port['id'])

        super(MiscPolicyActionsNetworkRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

        # Create network the interface will be attached to
        network_name = \
            data_utils.rand_name(cls.__class__.__name__ + '-network')
        post_body = {'name': network_name}
        post_body['router:external'] = False
        post_body['shared'] = True
        post_body['port_security_enabled'] = True
        cls.network = \
            cls.networks_client.create_network(**post_body)['network']
        cls.addClassResourceCleanup(
            cls.networks_client.delete_network,
            cls.network['id'])

        # Create subnet for network
        cls.cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
        cls.subnet = cls.subnets_client.create_subnet(
            network_id=cls.network['id'],
            cidr=cls.cidr,
            ip_version=4)['subnet']
        cls.addClassResourceCleanup(
            cls.subnets_client.delete_subnet,
            cls.subnet['id'])

        # ports on the network need to be deleted before the network can
        # be deleted
        cls.addClassResourceCleanup(_cleanup_ports, cls.network['id'])

    def _attach_interface_to_server(self):
        network_id = self.network['id']
        interface = self.interfaces_client.create_interface(
            self.server['id'], net_id=network_id)['interfaceAttachment']
        self.addCleanup(
            waiters.wait_for_interface_detach, self.interfaces_client,
            self.server['id'], interface['port_id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.interfaces_client.delete_interface,
            self.server['id'], interface['port_id'])
        waiters.wait_for_interface_status(
            self.interfaces_client, self.server['id'],
            interface['port_id'], 'ACTIVE')
        return interface

    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          "Interface attachment is not available.")
    @utils.requires_ext(extension='os-attach-interfaces', service='compute')
    @decorators.idempotent_id('ddf53cb6-4a0a-4e5a-91e3-6c32aaa3b9b6')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-attach-interfaces"])
    def test_list_interfaces(self):
        """Test list interfaces, part of os-attach-interfaces."""
        with self.override_role():
            self.interfaces_client.list_interfaces(self.server['id'])

    @decorators.idempotent_id('1b9cf7db-dc50-48a2-8eb9-8c25af5e934a')
    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          "Interface attachment is not available.")
    @utils.requires_ext(extension='os-attach-interfaces', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-attach-interfaces"])
    def test_show_interface(self):
        """Test show interfaces, part of os-attach-interfaces."""
        interface = self._attach_interface_to_server()
        with self.override_role():
            self.interfaces_client.show_interface(
                self.server['id'], interface['port_id'])

    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          "Interface attachment is not available.")
    @utils.requires_ext(extension='os-attach-interfaces', service='compute')
    @decorators.idempotent_id('d2d3a24d-4738-4bce-a287-36d664746cde')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-attach-interfaces:create"])
    def test_create_interface(self):
        """Test create interface, part of os-attach-interfaces."""
        network_id = self.network['id']
        with self.override_role():
            interface = self.interfaces_client.create_interface(
                self.server['id'], net_id=network_id)['interfaceAttachment']
        self.addCleanup(
            waiters.wait_for_interface_detach, self.interfaces_client,
            self.server['id'], interface['port_id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.interfaces_client.delete_interface,
            self.server['id'], interface['port_id'])
        waiters.wait_for_interface_status(
            self.interfaces_client, self.server['id'],
            interface['port_id'], 'ACTIVE')

    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          "Interface attachment is not available.")
    @utils.requires_ext(extension='os-attach-interfaces', service='compute')
    @decorators.idempotent_id('55b05692-ed44-4608-a84c-cd4219c82799')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-attach-interfaces:delete"])
    def test_delete_interface(self):
        """Test delete interface, part of os-attach-interfaces."""
        interface = self._attach_interface_to_server()

        with self.override_role():
            self.interfaces_client.delete_interface(self.server['id'],
                                                    interface['port_id'])
        waiters.wait_for_interface_detach(
            self.interfaces_client, self.server['id'], interface['port_id'])

    @decorators.idempotent_id('6886d360-0d86-4760-b1a3-882d81fbebcc')
    @utils.requires_ext(extension='os-ips', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:ips:index"])
    def test_list_addresses(self):
        """Test list server addresses, part of ips policy family."""
        with self.override_role():
            self.servers_client.list_addresses(self.server['id'])

    @decorators.idempotent_id('fa43e7e5-0db9-48eb-9c6b-c11eb766b8e4')
    @utils.requires_ext(extension='os-ips', service='compute')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:ips:show"])
    def test_list_addresses_by_network(self):
        """Test list server addresses by network, part of ips policy family."""
        addresses = self.servers_client.list_addresses(self.server['id'])[
            'addresses']
        address = next(iter(addresses))

        with self.override_role():
            self.servers_client.list_addresses_by_network(
                self.server['id'], address)

    @testtools.skipUnless(CONF.compute_feature_enabled.interface_attach,
                          "Interface attachment is not available.")
    @utils.requires_ext(extension='os-multinic', service='compute')
    @rbac_rule_validation.action(
        service="nova", rules=["os_compute_api:os-multinic"])
    @decorators.idempotent_id('bd3e2c74-130a-40f0-8085-124d93fe67da')
    def test_add_fixed_ip(self):
        """Test add fixed ip to server network, part of os-multinic."""
        interfaces = (self.interfaces_client.list_interfaces(self.server['id'])
                      ['interfaceAttachments'])
        if interfaces:
            network_id = interfaces[0]['net_id']
        else:
            interface = self.interfaces_client.create_interface(
                self.server['id'])['interfaceAttachment']
            network_id = interface['net_id']
            self.addCleanup(
                waiters.wait_for_interface_detach, self.interfaces_client,
                self.server['id'], interface['port_id'])
            self.addCleanup(
                self.interfaces_client.delete_interface,
                self.server['id'], interface['port_id'])

        with self.override_role():
            self.servers_client.add_fixed_ip(self.server['id'],
                                             networkId=network_id)
        # Get the Fixed IP from server.
        server_detail = self.servers_client.show_server(
            self.server['id'])['server']
        fixed_ip = None
        for ip_set in server_detail['addresses']:
            for ip in server_detail['addresses'][ip_set]:
                if ip['OS-EXT-IPS:type'] == 'fixed':
                    fixed_ip = ip['addr']
                    break
            if fixed_ip is not None:
                break
        # Remove the fixed IP from server.
        self.servers_client.remove_fixed_ip(self.server['id'],
                                            address=fixed_ip)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class PolicyBandwidthLimitRuleExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(PolicyBandwidthLimitRuleExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('qos', 'network'):
            msg = "qos extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(PolicyBandwidthLimitRuleExtRbacTest, cls).resource_setup()
        name = data_utils.rand_name(cls.__class__.__name__ + '-qos-policy')
        cls.policy_id = cls.ntp_client.create_qos_policy(
            name=name)["policy"]["id"]
        cls.addClassResourceCleanup(cls.ntp_client.delete_qos_policy,
                                    cls.policy_id)

    def _create_bandwidth_limit_rule(self):
        rule = self.ntp_client.create_bandwidth_limit_rule(
            self.policy_id, max_kbps=1000, max_burst_kbps=1000,
            direction="egress")
        rule_id = rule['bandwidth_limit_rule']['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ntp_client.delete_bandwidth_limit_rule,
                        self.policy_id, rule_id)
        return rule_id

    @decorators.idempotent_id('E0FDCB39-E16D-4AF5-9165-3FEFD116E69D')
    @rbac_rule_validation.action(
        service="neutron", rules=["create_policy_bandwidth_limit_rule"])
    def test_create_policy_bandwidth_limit_rule(self):
        """Create bandwidth_limit_rule.

        RBAC test for the neutron "create_policy_bandwidth_limit_rule" policy
        """

        with self.override_role():
            self._create_bandwidth_limit_rule()

    @decorators.idempotent_id('A092BD50-364F-4F55-B81A-37DAD6E77B95')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_bandwidth_limit_rule"],
                                 expected_error_codes=[404])
    def test_show_policy_bandwidth_limit_rule(self):
        """Show bandwidth_limit_rule.

        RBAC test for the neutron "get_policy_bandwidth_limit_rule" policy
        """
        rule_id = self._create_bandwidth_limit_rule()

        with self.override_role():
            self.ntp_client.show_bandwidth_limit_rule(self.policy_id, rule_id)

    @decorators.idempotent_id('CAA27599-082B-44B9-AF09-8C9B8E777ED7')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_bandwidth_limit_rule",
                                        "update_policy_bandwidth_limit_rule"],
                                 expected_error_codes=[404, 403])
    def test_update_policy_bandwidth_limit_rule(self):
        """Update bandwidth_limit_rule.

        RBAC test for the neutron "update_policy_bandwidth_limit_rule" policy
        """
        rule_id = self._create_bandwidth_limit_rule()

        with self.override_role():
            self.ntp_client.update_bandwidth_limit_rule(
                self.policy_id, rule_id, max_kbps=2000)

    @decorators.idempotent_id('BF6D9ED7-4B04-423D-857D-455DB0705852')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_policy_bandwidth_limit_rule",
                                        "delete_policy_bandwidth_limit_rule"],
                                 expected_error_codes=[404, 403])
    def test_delete_policy_bandwidth_limit_rule(self):
        """Delete bandwidth_limit_rule.

        RBAC test for the neutron "delete_policy_bandwidth_limit_rule" policy
        """
        rule_id = self._create_bandwidth_limit_rule()

        with self.override_role():
            self.ntp_client.delete_bandwidth_limit_rule(self.policy_id,
                                                        rule_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityTokenV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def resource_setup(cls):
        super(IdentityTokenV3RbacTest, cls).resource_setup()
        cls.user_id = cls.os_primary.auth_provider.credentials.user_id
        cls.password = cls.os_primary.auth_provider.credentials.password

    @decorators.idempotent_id('201e2fe5-2023-4bce-9189-78b51520a91e')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:validate_token"],
        extra_target_data={
            "target.token.user_id":
            "os_primary.auth_provider.credentials.user_id"
        })
    def test_show_token(self):
        token_id = self.setup_test_token(self.user_id, self.password)
        with self.override_role():
            self.identity_client.show_token(token_id)

    @decorators.idempotent_id('42a299db-fe0a-4ea0-9824-0bfd13155886')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:revoke_token"],
        extra_target_data={
            "target.token.user_id":
            "os_primary.auth_provider.credentials.user_id"
        })
    def test_delete_token(self):
        token_id = self.setup_test_token(self.user_id, self.password)
        with self.override_role():
            self.identity_client.delete_token(token_id)

    @decorators.idempotent_id('3554d218-8cd6-4730-a1b2-0e22f9b78f45')
    @rbac_rule_validation.action(
        service="keystone",
        rules=["identity:check_token"],
        extra_target_data={
            "target.token.user_id":
            "os_primary.auth_provider.credentials.user_id"
        })
    def test_check_token_exsitence(self):
        token_id = self.setup_test_token(self.user_id, self.password)
        with self.override_role():
            self.identity_client.check_token_existence(token_id)



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
# Copyright 2017 AT&T Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.tests import base

from patrole_tempest_plugin.tests.unit import fixtures as patrole_fixtures


class TestCase(base.TestCase):
    """Test case base class for all unit tests."""

    def setUp(self):
        super(TestCase, self).setUp()
        # Disable patrole log for unit tests.
        self.useFixture(
            patrole_fixtures.ConfPatcher(enable_reporting=False,
                                         group='patrole_log'))



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class QuotaClassesV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(QuotaClassesV3RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-quota-class-sets', 'volume'):
            msg = ("%s skipped as os-quota-class-sets not enabled."
                   % cls.__name__)
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(QuotaClassesV3RbacTest, cls).setup_clients()
        cls.quota_classes_client = cls.os_primary.quota_classes_client
        cls.quota_name = data_utils.rand_name(cls.__name__ + '-QuotaClass')

    @decorators.idempotent_id('1a060def-2b43-4534-97f5-5eadbbe8c726')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume_extension:quota_classes"])
    def test_show_quota_class_set(self):
        with self.override_role():
            self.quota_classes_client.show_quota_class_set(
                self.quota_name)['quota_class_set']

    @decorators.idempotent_id('72159478-23a7-4c75-989f-6bac609eca62')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume_extension:quota_classes"])
    def test_update_quota_class_set(self):
        quota_class_set = self.quota_classes_client.show_quota_class_set(
            self.quota_name)['quota_class_set']
        quota_class_set.pop('id')

        with self.override_role():
            self.quota_classes_client.update_quota_class_set(self.quota_name,
                                                             **quota_class_set)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityConsumersV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    def _create_consumer(self):
        description = data_utils.rand_name(
            self.__class__.__name__ + '-IdentityConsumer')
        consumer = self.consumers_client.create_consumer(
            description)['consumer']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.consumers_client.delete_consumer,
                        consumer['id'])
        return consumer

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_consumer"])
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d970')
    def test_create_consumer(self):
        with self.override_role():
            self._create_consumer()

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_consumer"])
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d971')
    def test_delete_consumer(self):
        consumer = self._create_consumer()

        with self.override_role():
            self.consumers_client.delete_consumer(consumer['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_consumer"])
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d972')
    def test_update_consumer(self):
        consumer = self._create_consumer()
        updated_description = data_utils.rand_name(
            self.__class__.__name__ + '-IdentityConsumer')

        with self.override_role():
            self.consumers_client.update_consumer(consumer['id'],
                                                  updated_description)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_consumer"])
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d973')
    def test_show_consumer(self):
        consumer = self._create_consumer()

        with self.override_role():
            self.consumers_client.show_consumer(consumer['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_consumers"])
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d975')
    def test_list_consumers(self):
        with self.override_role():
            self.consumers_client.list_consumers()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumesSnapshotV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(VolumesSnapshotV3RbacTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesSnapshotV3RbacTest, cls).resource_setup()
        # Create a test shared volume for tests
        cls.volume = cls.create_volume()
        # Create a test shared snapshot for tests
        cls.snapshot = cls.create_snapshot(cls.volume['id'])

    def _list_by_param_values(self, with_detail=False, **params):
        # Perform list or list_details action with given params.
        return self.snapshots_client.list_snapshots(
            detail=with_detail, **params)['snapshots']

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:create_snapshot"])
    @decorators.idempotent_id('ac7b2ee5-fbc0-4360-afc2-de8fa4881ede')
    def test_create_snapshot(self):
        # Create a temp snapshot
        with self.override_role():
            self.create_snapshot(self.volume['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_snapshot"])
    @decorators.idempotent_id('93a11b40-1ba8-44d6-a196-f8d97220f796')
    def test_show_snapshot(self):
        # Get the snapshot
        with self.override_role():
            self.snapshots_client.show_snapshot(
                self.snapshot['id'])['snapshot']

    @decorators.idempotent_id('5d6f5f21-9293-4f2a-8f44-cabdc24d92cb')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:extended_snapshot_attributes"])
    def test_show_snapshot_with_extended_attributes(self):
        """List snapshots with extended attributes."""
        expected_attrs = ('os-extended-snapshot-attributes:project_id',
                          'os-extended-snapshot-attributes:progress')

        with self.override_role():
            resp = self.snapshots_client.show_snapshot(
                self.snapshot['id'])['snapshot']
        for expected_attr in expected_attrs:
            if expected_attr not in resp:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=expected_attr)

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:update_snapshot"])
    @decorators.idempotent_id('53fe8ee3-3bea-4ae8-a979-3c98ea72f620')
    def test_update_snapshot(self):
        new_desc = 'This is the new description of snapshot.'
        params = {'description': new_desc}
        # Updates snapshot with new values
        with self.override_role():
            self.snapshots_client.update_snapshot(
                self.snapshot['id'], **params)['snapshot']
        waiters.wait_for_volume_resource_status(
            self.snapshots_client, self.snapshot['id'], 'available')

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:delete_snapshot"])
    @decorators.idempotent_id('c7fe54ec-3b70-4772-ba11-f166d95888a3')
    def test_delete_snapshot(self):
        # Create a temp snapshot
        temp_snapshot = self.create_snapshot(self.volume['id'])
        with self.override_role():
            # Delete the snapshot
            self.snapshots_client.delete_snapshot(temp_snapshot['id'])
        self.snapshots_client.wait_for_resource_deletion(
            temp_snapshot['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_all_snapshots"])
    @decorators.idempotent_id('e4edf0c0-2cd3-420f-b8ab-4d98a0718608')
    def test_list_snapshots(self):
        """List snapshots with params."""
        params = {'name': self.snapshot['name']}
        with self.override_role():
            self._list_by_param_values(**params)

    @decorators.idempotent_id('f3155d8e-45ee-45c9-910d-18c0242229e1')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_all_snapshots"])
    def test_list_snapshots_details(self):
        """List snapshots details with params."""
        params = {'name': self.snapshot['name']}
        with self.override_role():
            self._list_by_param_values(with_detail=True, **params)

    @decorators.idempotent_id('dd37f388-2731-446d-a78f-676997ebb04a')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:extended_snapshot_attributes"])
    def test_list_snapshots_details_with_extended_attributes(self):
        """List snapshots details with extended attributes."""
        expected_attrs = ('os-extended-snapshot-attributes:project_id',
                          'os-extended-snapshot-attributes:progress')
        params = {'name': self.snapshot['name']}

        with self.override_role():
            resp = self._list_by_param_values(with_detail=True, **params)
        for expected_attr in expected_attrs:
            if expected_attr not in resp[0]:
                raise rbac_exceptions.RbacMissingAttributeResponseBody(
                    attribute=expected_attr)



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from tempest.common import utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class SegmentsExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(SegmentsExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('segment', 'network'):
            msg = "segment extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SegmentsExtRbacTest, cls).resource_setup()
        cls.network = cls.create_network()

    @classmethod
    def get_free_segmentation_id(cls):
        # Select unused segmentation_id to prevent usage conflict
        segments = cls.ntp_client.list_segments()["segments"]
        segmentation_ids = [s["segmentation_id"] for s in segments]

        # With 2+ concurrency, tests that ran in the same moment may fail due
        # to usage conflict. To prevent it we select segmentation to start
        # randomly.
        segmentation_id = random.randint(1000, 5000)  # nosec
        while segmentation_id in segmentation_ids:
            segmentation_id += 1

        return segmentation_id

    @classmethod
    def create_segment(cls, network):
        segmentation_id = cls.get_free_segmentation_id()

        seg = cls.ntp_client.create_segment(
            network_id=network['id'], network_type="gre",
            segmentation_id=segmentation_id)
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.ntp_client.delete_segment, seg['segment']['id'])

        return seg

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08126')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_segment"])
    def test_create_segment(self):
        """Create segment.

        RBAC test for the neutron "create_segment" policy
        """
        with self.override_role():
            self.create_segment(self.network)

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08127')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_segment"],
                                 expected_error_codes=[404])
    def test_show_segment(self):
        """Show segment.

        RBAC test for the neutron "get_segment" policy
        """
        segment = self.create_segment(self.network)

        with self.override_role():
            self.ntp_client.show_segment(segment['segment']['id'])

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08128')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_segment",
                                        "update_segment"],
                                 expected_error_codes=[404, 403])
    def test_update_segment(self):
        """Update segment.

        RBAC test for the neutron "update_segment" policy
        """
        segment = self.create_segment(self.network)

        with self.override_role():
            self.ntp_client.update_segment(segment['segment']['id'],
                                           name="NewName")

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08129')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_segment",
                                        "delete_segment"],
                                 expected_error_codes=[404, 403])
    def test_delete_segment(self):
        """Delete segment.

        RBAC test for the neutron "delete_segment" policy
        """
        segment = self.create_segment(self.network)

        with self.override_role():
            self.ntp_client.delete_segment(segment['segment']['id'])

    @decorators.idempotent_id('d68a0578-36ae-435e-8aaa-508ee96bdfae')
    @rbac_rule_validation.action(service="neutron", rules=["get_segment"])
    def test_list_segments(self):
        """List segments.

        RBAC test for the neutron ``list_segments`` function and
        the``get_segment`` policy
        """
        admin_resource_id = self.create_segment(self.network)['segment']['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.ntp_client.list_segments(
                id=admin_resource_id)["segments"]



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import base64
from oslo_serialization import jsonutils as json

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumesBackupsV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsV3RbacTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesBackupsV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()
        cls.backup = cls._create_backup(volume_id=cls.volume['id'])

    def _decode_url(self, backup_url):
        return json.loads(base64.decode_as_text(backup_url))

    def _encode_backup(self, backup):
        retval = json.dumps(backup)
        return base64.encode_as_text(retval)

    def _modify_backup_url(self, backup_url, changes):
        backup = self._decode_url(backup_url)
        backup.update(changes)
        return self._encode_backup(backup)

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:create"])
    @decorators.idempotent_id('6887ec94-0bcf-4ab7-b30f-3808a4b5a2a5')
    def test_create_backup(self):
        backup_name = data_utils.rand_name(self.__class__.__name__ + '-Backup')

        with self.override_role():
            backup = self.backups_client.create_backup(
                volume_id=self.volume['id'], name=backup_name)['backup']
        self.addCleanup(self.backups_client.delete_backup, backup['id'])
        waiters.wait_for_volume_resource_status(
            self.backups_client, backup['id'], 'available')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:get"])
    @decorators.idempotent_id('abd92bdd-b0fb-4dc4-9cfc-de9e968f8c8a')
    def test_show_backup(self):
        with self.override_role():
            self.backups_client.show_backup(self.backup['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:get_all"])
    @decorators.idempotent_id('4d18f0f0-7e01-4007-b622-dedc859b22f6')
    def test_list_backups(self):
        with self.override_role():
            self.backups_client.list_backups()

    @decorators.idempotent_id('dbd69865-876f-4835-b70e-7341153fb162')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:get_all"])
    def test_list_backups_with_details(self):
        with self.override_role():
            self.backups_client.list_backups(detail=True)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('50f43bde-205e-438e-9a05-5eac07fc3d63')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:backup_admin_actions:reset_status"])
    def test_reset_backup_status(self):
        # Use instance-level create_backup for easier debugging.
        backup = self.create_backup(volume_id=self.volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

        with self.override_role():
            self.backups_client.reset_backup_status(backup_id=backup['id'],
                                                    status='error')
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                backup['id'], 'error')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:restore"])
    @decorators.idempotent_id('9c794bf9-2446-4f41-8fe0-80b71e757f9d')
    def test_restore_backup(self):
        with self.override_role():
            restore = self.backups_client.restore_backup(
                self.backup['id'])['restore']
        self.addCleanup(self.volumes_client.delete_volume,
                        restore['volume_id'])
        waiters.wait_for_volume_resource_status(
            self.backups_client, restore['backup_id'], 'available')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:delete"])
    @decorators.idempotent_id('d5d0c6a2-413d-437e-a73f-4bf2b41a20ed')
    def test_delete_backup(self):
        # Do not call the create_backup in Tempest's base volume class, because
        # it doesn't use ``test_utils.call_and_ignore_notfound_exc`` for clean
        # up.
        backup = self.backups_client.create_backup(
            volume_id=self.volume['id'])['backup']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.backups_client.delete_backup, backup['id'])
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                backup['id'], 'available')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

        with self.override_role():
            self.backups_client.delete_backup(backup['id'])
        # Wait for deletion so error isn't thrown during clean up.
        self.backups_client.wait_for_resource_deletion(backup['id'])

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:export-import"])
    @decorators.idempotent_id('e984ec8d-e8eb-485c-98bc-f1856020303c')
    def test_export_backup(self):
        with self.override_role():
            self.backups_client.export_backup(self.backup['id'])[
                'backup-record']

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:backup-import"])
    @decorators.idempotent_id('1e70f039-4556-44cc-9cc1-edf2b7ed648b')
    def test_import_backup(self):
        export_backup = self.backups_client.export_backup(
            self.backup['id'])['backup-record']
        new_id = data_utils.rand_uuid()
        new_url = self._modify_backup_url(
            export_backup['backup_url'], {'id': new_id})

        with self.override_role():
            import_backup = self.backups_client.import_backup(
                backup_service=export_backup['backup_service'],
                backup_url=new_url)['backup']
        self.addCleanup(self.backups_client.delete_backup, import_backup['id'])
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                import_backup['id'],
                                                'available')


class VolumesBackupsV318RbacTest(rbac_base.BaseVolumeRbacTest):
    """Validates that the ``GET /backups/{backup_id}`` and
    ``GET /backups/details`` APIs inject the expected attribute
    'os-backup-project-attr:project_id' into the response body following
    successful authorization.
    """
    _api_version = 3
    # The minimum microversion for showing 'os-backup-project-attr:project_id'
    # is 3.18.
    min_microversion = '3.18'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsV318RbacTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesBackupsV318RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()
        cls.backup = cls._create_backup(volume_id=cls.volume['id'])
        cls.expected_attr = 'os-backup-project-attr:project_id'

    @decorators.idempotent_id('69801485-d5be-4e75-bbb4-168d50b5a8c2')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:backup_project_attribute"])
    def test_show_backup_project_attribute(self):
        with self.override_role():
            body = self.backups_client.show_backup(self.backup['id'])['backup']

        # Show backup API attempts to inject the attribute below into the
        # response body but only if policy enforcement succeeds.
        if self.expected_attr not in body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=self.expected_attr)

    @decorators.idempotent_id('aa40b7c0-5974-48be-8cbc-e23cc61c4c68')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["backup:backup_project_attribute"])
    def test_list_backup_details_project_attribute(self):
        with self.override_role():
            body = self.backups_client.list_backups(detail=True)['backups']

        if self.expected_attr not in body[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=self.expected_attr)


class VolumesBackupsV39RbacTest(rbac_base.BaseVolumeRbacTest):
    _api_version = 3
    min_microversion = '3.9'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsV39RbacTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesBackupsV39RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()
        cls.backup = cls._create_backup(volume_id=cls.volume['id'])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('b45b0e98-6eb8-4c62-aa53-0f8c7c09faa6')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["backup:update"])
    def test_backup_update(self):
        update_kwargs = {
            'name': data_utils.rand_name(self.__class__.__name__ + '-Backup'),
            'description': data_utils.rand_name("volume-backup-description")
        }
        with self.override_role():
            self.backups_client.update_backup(self.backup['id'],
                                              **update_kwargs)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from patrole_tempest_plugin import rbac_utils

LOG = logging.getLogger(__name__)


class BaseIdentityRbacTest(rbac_utils.RbacUtilsMixin,
                           base.BaseIdentityTest):

    @classmethod
    def setup_test_endpoint(cls, service=None):
        """Creates a service and an endpoint for test."""
        interface = 'public'
        url = data_utils.rand_url()
        region_name = data_utils.rand_name(
            cls.__name__ + '-region')
        # Endpoint creation requires a service
        if service is None:
            service = cls.setup_test_service()
        params = {
            'service_id': service['id'],
            'region': region_name,
            'interface': interface,
            'url': url
        }

        endpoint = cls.endpoints_client.create_endpoint(**params)['endpoint']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.endpoints_client.delete_endpoint, endpoint['id'])

        return endpoint

    @classmethod
    def setup_test_role(cls):
        """Set up a test role."""
        name = data_utils.rand_name(cls.__name__ + '-test_role')
        role = cls.roles_client.create_role(name=name)['role']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.roles_client.delete_role, role['id'])

        return role

    @classmethod
    def setup_test_service(cls):
        """Setup a test service."""
        name = data_utils.rand_name(cls.__name__ + '-service')
        serv_type = data_utils.rand_name('type')
        desc = data_utils.rand_name(cls.__name__ + '-description')

        service = cls.services_client.create_service(
            name=name,
            type=serv_type,
            description=desc)['service']

        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.services_client.delete_service, service['id'])

        return service

    @classmethod
    def setup_test_user(cls, password=None, **kwargs):
        """Set up a test user."""
        username = data_utils.rand_name(cls.__name__ + '-test_user')
        email = username + '@testmail.tm'

        user = cls.users_client.create_user(
            name=username,
            email=email,
            password=password,
            **kwargs)['user']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.users_client.delete_user, user['id'])

        return user


class BaseIdentityV3RbacTest(BaseIdentityRbacTest):

    identity_version = 'v3'
    credentials = ['primary']

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityV3RbacTest, cls).setup_clients()
        cls.application_credentials_client = \
            cls.os_primary.application_credentials_client
        cls.creds_client = cls.os_primary.credentials_client
        cls.consumers_client = cls.os_primary.oauth_consumers_client
        cls.domains_client = cls.os_primary.domains_client
        cls.domain_config_client = cls.os_primary.domain_config_client
        cls.endpoints_client = cls.os_primary.endpoints_v3_client
        cls.endpoint_filter_client = cls.os_primary.endpoint_filter_client
        cls.endpoint_groups_client = cls.os_primary.endpoint_groups_client
        cls.groups_client = cls.os_primary.groups_client
        cls.identity_client = cls.os_primary.identity_v3_client
        cls.oauth_token_client = cls.os_primary.oauth_token_client
        cls.projects_client = cls.os_primary.projects_client
        cls.project_tags_client = cls.os_primary.project_tags_client
        cls.policies_client = cls.os_primary.policies_client
        cls.regions_client = cls.os_primary.regions_client
        cls.role_assignments_client = cls.os_primary.role_assignments_client
        cls.roles_client = cls.os_primary.roles_v3_client
        cls.services_client = cls.os_primary.identity_services_v3_client
        cls.token_client = cls.os_primary.token_v3_client
        cls.trusts_client = cls.os_primary.trusts_client
        cls.users_client = cls.os_primary.users_v3_client

    @classmethod
    def resource_setup(cls):
        super(BaseIdentityV3RbacTest, cls).resource_setup()
        cls.credentials = []
        cls.domains = []
        cls.groups = []
        cls.policies = []
        cls.projects = []
        cls.regions = []
        cls.trusts = []
        cls.tokens = []

    @classmethod
    def resource_cleanup(cls):
        for credential in cls.credentials:
            test_utils.call_and_ignore_notfound_exc(
                cls.creds_client.delete_credential, credential['id'])

        # Delete each domain at the end of the test, but each domain must be
        # disabled first.
        for domain in cls.domains:
            test_utils.call_and_ignore_notfound_exc(
                cls.domains_client.update_domain, domain['id'], enabled=False)
            test_utils.call_and_ignore_notfound_exc(
                cls.domains_client.delete_domain, domain['id'])

        for group in cls.groups:
            test_utils.call_and_ignore_notfound_exc(
                cls.groups_client.delete_group, group['id'])

        for policy in cls.policies:
            test_utils.call_and_ignore_notfound_exc(
                cls.policies_client.delete_policy, policy['id'])

        for project in cls.projects:
            test_utils.call_and_ignore_notfound_exc(
                cls.projects_client.delete_project, project['id'])

        for region in cls.regions:
            test_utils.call_and_ignore_notfound_exc(
                cls.regions_client.delete_region, region['id'])

        for trust in cls.trusts:
            test_utils.call_and_ignore_notfound_exc(
                cls.trusts_client.delete_trust, trust['id'])

        for token in cls.tokens:
            test_utils.call_and_ignore_notfound_exc(
                cls.identity_client.delete_token, token)

        super(BaseIdentityV3RbacTest, cls).resource_cleanup()

    @classmethod
    def setup_test_credential(cls, user=None):
        """Creates a credential for test."""
        keys = [data_utils.rand_uuid_hex(),
                data_utils.rand_uuid_hex()]
        blob = '{"access": "%s", "secret": "%s"}' % (keys[0], keys[1])

        credential = cls.creds_client.create_credential(
            user_id=user['id'],
            project_id=user['project_id'],
            blob=blob,
            type='ec2')['credential']
        cls.credentials.append(credential)

        return credential

    @classmethod
    def setup_test_domain(cls):
        """Set up a test domain."""
        domain = cls.domains_client.create_domain(
            name=data_utils.rand_name(cls.__name__ + '-test_domain'),
            description=data_utils.rand_name(
                cls.__name__ + '-desc'))['domain']
        cls.domains.append(domain)

        return domain

    @classmethod
    def setup_test_group(cls):
        """Creates a group for test."""
        name = data_utils.rand_name(cls.__name__ + '-test_group')
        group = cls.groups_client.create_group(name=name)['group']
        cls.groups.append(group)

        return group

    @classmethod
    def setup_test_policy(cls):
        """Creates a policy for test."""
        blob = data_utils.rand_name(cls.__name__ + '-test_blob')
        policy_type = data_utils.rand_name(
            cls.__name__ + '-policy_type')

        policy = cls.policies_client.create_policy(
            blob=blob,
            policy=policy_type,
            type="application/json")['policy']
        cls.policies.append(policy)

        return policy

    @classmethod
    def setup_test_project(cls):
        """Set up a test project."""
        project = cls.projects_client.create_project(
            name=data_utils.rand_name(
                cls.__name__ + '-testproject'),
            description=data_utils.rand_name(
                cls.__name__ + '-desc'))['project']
        cls.projects.append(project)

        return project

    @classmethod
    def setup_test_region(cls):
        """Creates a region for test."""
        description = data_utils.rand_name(
            cls.__name__ + '-test_region_desc')

        region = cls.regions_client.create_region(
            description=description)['region']
        cls.regions.append(region)

        return region

    @classmethod
    def setup_test_trust(cls, trustee_user_id, trustor_user_id, **kwargs):
        """Setup a test trust."""
        trust = cls.trusts_client.create_trust(
            trustee_user_id=trustee_user_id, trustor_user_id=trustor_user_id,
            impersonation=False, **kwargs)['trust']
        cls.trusts.append(trust)

        return trust

    @classmethod
    def setup_test_token(cls, user_id, password):
        """Set up a test token."""
        token = cls.token_client.auth(user_id=user_id,
                                      password=password).response
        token_id = token['x-subject-token']
        cls.tokens.append(token_id)
        return token_id
